#!/usr/bin/perl -w
#< Boilerplate
#
# The Aranea question answering system
# Copyright (C) 2002-2005, Jimmy Lin
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, please visit
# http://www.gnu.org/copyleft/gpl.html
#
# $Header: /fs/clip-qa/.cvsroot//Aranea/pm/Aranea/Common.pm,v 1.18 2005/06/08 02:51:42 jimmylin Exp $
#
#>
#< POD

###############################################################################

=head1 NAME

B<C<Aranea::Common>> - library of common procedures

=head1 SYNOPSIS

  my $content = safe_get("http://www.ai.mit.edu/index.html");
  my $safe = demoronize($content);
  my $tagged = pos_tag('This is a test');

=head1 DESCRIPTION

This module contains commonly used procedures shared by all Aranea
components:

=over 4

=cut

###############################################################################

#>
#< Header

package Aranea::Common;

use strict;
use warnings;

use LWP::UserAgent;
use HTTP::Request;

use LPost;

use Exporter;

our @ISA = qw(Exporter);
our @EXPORT = qw();
our @EXPORT_OK = qw(demoronize strip_pos_tags safe_get pos_tag find_file);
our %EXPORT_TAGS = ( 'default' => [ qw(demoronize strip_pos_tags safe_get pos_tag find_file) ] );

our @UserAgent;
our $UA;
our $PAGE_DELAY = 30;
our $TIMEOUT = 180;

#>

#< BEGIN
BEGIN {
    LPost::initialize_tagger();

    # various UserAgent strings correponding to different browsers and
    # different platforms
    @UserAgent = ('Mozilla/4.5 [en] (X11; I; Linux 2.2.0-pre4 i586; Nav)',
		     'Mozilla/4.73 [en] (X11; U; Linux 2.2.14 i586)',
		     'Mozilla/4.73 [en] (X11; U; Linux 2.2.14 i686)',
		     'Mozilla/4.73 [en] (X11; U; Linux 2.2.15 i686)',
		     'Mozilla/4.74 [en] (X11; U; Linux 2.2.14-5.0 i686)',
		     'Mozilla/4.74 [en] (X11; U; Linux 2.2.16 i586)',
		     'Mozilla/4.74 [en] (X11; U; Linux 2.2.16 i686)',
		     'Mozilla/5.0 (Linux 2.4.12 i686; U) Opera 5.0  [en]',
		     'Opera/5.0 (Linux 2.2.12 i386; U)  [en]',
		     'Opera/5.0 (Linux 2.2.14 i686; U)  [en]',
		     'Opera/5.0 (Linux 2.2.14-5.0 i686; U)  [en]',
		     'Mozilla/4.72 [en] (X11; U; Linux 2.2.19 i686)',
		     'Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1)',
		     'Mozilla/4.0 (compatible; MSIE 5.5; Windows NT 5.0)');

    $UA = new LWP::UserAgent;

    # prepare a log file
    open(LOG, ">> /tmp/web_proxy.log") or
	die "Unable to open log file at /tmp/web_proxy.log";

    # make sure the cache directory exists.
    unless ( -e "/tmp/web_cache/" ) {
	print LOG "Cache directory doesn't exist.  " .
	    "Creating /tmp/web_cache now.\n";
	mkdir("/tmp/web_cache/");
    }

    close(LOG);

}
#>

#< sub set_time_token
sub set_time_token {
    open(TOKEN, "> /tmp/aranea.timetoken") or
	print STDERR "Unable to open token: /tmp/aranea.timetoken";

    my ($sec, $min, $hour, $mday,
	$mon, $year, $wday, $yday, $isdst) = localtime(time);

    my $tkn = $year + $yday*24*3600 + $hour*3600 + $min*60 + $sec;

    print TOKEN $tkn;
    return $tkn;
}
#>
#< sub get_time_token
sub get_time_token {
  if ( -e "/tmp/aranea.timetoken" ) {
    open(TOKEN, "< /tmp/aranea.timetoken");
    my $tkn = join('', <TOKEN>);
    close(TOKEN);

    return $tkn;
  }

  return -1;
}
#>
#< sub get_time_in_seconds
sub get_time_in_seconds {
    my ($sec, $min, $hour, $mday,
	$mon, $year, $wday, $yday, $isdst) = localtime(time);

    return $year + $yday*24*3600 + $hour*3600 + $min*60 + $sec;
}
#>
#< sub safe_get

=item B<C<safe_get($url)>>

This procedure serves as a replacement of C<get> in C<LWP::Simple>,
and alleviates the problem bombarding external sites with non-stop
HTTP requests (which often gets us banned from those sites).

The procedure enforces a mandatory delay between retrieval of live Web
pages (cached pages can be retrieved instantly).  The default is 30
seconds, but can be adjusted using C<safe_delay>.

When fetching live pages, the procedure randomly selects a UserAgent
string for its C<GET> request, emulating a variety of browsers:
Netscape, IE, Opera (and different versions thereof.)  Furthermore,
Web pages are cached in C</tmp/web_cache>.  The cache can be cleared
by simply deleting any files there.

=cut
sub safe_get {
    my $query = shift;

    open(LOG, ">> /tmp/web_proxy.log") or
	print STDERR "Unable to open log file at /tmp/web_proxy.log";

    $query =~ s/[\r\n]+$//;

    my $filename = $query;
    $filename =~ s/[\/+?*=:&]//g;
    $filename =~ s/ +$//g;    # kill trailing spaces
    $filename =~ s/^ +//g;    # kill leading spaces

    print LOG "Request at " . `date`;
    print LOG "Query = $query\n";
    print LOG "Looking for cached file '$filename'\n";

    if ( -e "/tmp/web_cache/$filename" ) {
	# cached copy exists, so serve the cached copy
	print LOG "Cached copy exists.\n";

	open(CACHE, "< /tmp/web_cache/$filename");
	my $raw = join('', <CACHE>);
	return $raw;

	close(CACHE);
    } else {
	# no cached copy, so forced to fetch page over web
	print LOG "Fetching live copy.\n";
	my $ua_string = $UserAgent[rand @UserAgent];
	print LOG "Using UserAgent = '$ua_string'\n";

	$UA->agent($ua_string);
	$UA->timeout($TIMEOUT);
	my $request = new HTTP::Request('GET', $query);

	my $response = $UA->request($request);
	my $content = $response->content;

	print LOG "Caching page to /tmp/web_cache/$filename\n";
	open(CACHE, "> /tmp/web_cache/$filename") or 
	    die "Couldn't write to cache!  Do I have write permission?\n";
	print CACHE $content;
	close(CACHE);

	# mandatory delay
	sleep($PAGE_DELAY);
	return $content;

    }

    return;
}
#>
#< sub set_page_delay
sub set_page_delay {
  $PAGE_DELAY = shift;
}
#>
#< sub get_page_delay
sub get_page_delay {
  return $PAGE_DELAY;
}
#>

#< sub set_timeout
sub set_timeout {
    $TIMEOUT = shift;
}
#>

#< sub get_timeout
sub get_timeout {
    return $TIMEOUT;
}
#>

#< sub demoronize

=item B<C<demoronize($str)>>

Fixes moronic Microsoft-generated encodings.  Microsoft-generated Web
pages use an extended version of ASCII that includes things like
special open/close quotes, I<e> with an accent mark, and other weird
letters.  True to Microsoft, this encoding is not compatable with
Unicode/utf-8, so it breaks the XML parser.  This procedure maps the
moronic Microsoft characters into normal ASCII characters.  The
original code for the demoroniser can be found at
L<http://www.fourmilab.ch/webtools/demoroniser/>.

=cut

# Since the original demoroniser was released into the public domain,
# there is no problem with stealing the code and including it here.

sub demoronize {
    my ($content) = @_;

    #   Map strategically incompatible non-ISO characters in the
    #   range 0x82 -- 0x9F into plausible substitutes where
    #   possible.

    $content =~ s/\x82/,/g;
    $content =~ s-\x83-<em>f</em>-g;
    $content =~ s/\x84/,,/g;
    $content =~ s/\x85/.../g;

    $content =~ s/\x88/^/g;
    $content =~ s-\x89- /-g;
    $content =~ s/\x8B/</g;
    $content =~ s/\x8C/Oe/g;

    $content =~ s/\x91/\`/g; 
    $content =~ s/\x92/\'/g;
    $content =~ s/\x93/\`\`/g;
    $content =~ s/\x94/\'\'/g;
    $content =~ s/\x95/*/g; 
    $content =~ s/\x96/-/g; 
    $content =~ s/\x97/--/g;
    $content =~ s-\x98-<sup>~</sup>-g;
    $content =~ s-\x99-<sup>TM</sup>-g;

    $content =~ s/\x9B/>/g;
    $content =~ s/\x9C/oe/g;

    # the above mappings were stolen from demoronizer; the ones below
    # were added later.

    $content =~ s/\216/Z/g;
    $content =~ s/\232/s/g;
    $content =~ s/\236/z/g;
    $content =~ s/\237/Y/g;

    $content =~ s/[\300-\305]/A/g;
    $content =~ s/\306/AE/g;
    $content =~ s/\307/C/g;
    $content =~ s/[\310-\313]/E/g;
    $content =~ s/[\314-\317]/I/g;
    $content =~ s/\320/D/g;
    $content =~ s/\321/N/g;
    $content =~ s/[\322-\326]/O/g;
    $content =~ s/[\331-\334]/U/g;
    $content =~ s/\335/Y/g;
    $content =~ s/\337/ss/g;
    $content =~ s/[\340-\345]/a/g;
    $content =~ s/\346/ae/g;
    $content =~ s/\347/c/g;
    $content =~ s/[\350-\353]/e/g;
    $content =~ s/[\354-\357]/i/g;
    $content =~ s/\361/n/g;
    $content =~ s/[\362-\366]/o/g;
    $content =~ s/[\371-\374]/u/g;
    $content =~ s/\375/y/g;
    $content =~ s/\377/y/g;

    # just convert everything else to spaces
    $content =~ s/[\200-\377]/ /g;

    return $content;
}

#>
#< sub pos_tag

=item B<C<pos_tag($str)>>

Part-of-speech tags the argument (using C<LPost>).  To amortize
startup costs, all tagging should be performed through this procedure
(instead of using LPost directly).

=cut

sub pos_tag {
    return LPost::tag(shift);
}
#>
#< sub find_file
sub find_file {
    my $relpath = shift;

    foreach my $path ( @INC ) {
	if ( -e "$path/$relpath" ) {
	    return "$path/$relpath";
	}
    }

    return '';
}
#>
#< sub strip_pos_tags
sub strip_pos_tags {
    my $s = shift;

    $s =~ s/\/[^ ]+//g;
    $s =~ s/^\W+//g;
    $s =~ s/\W+$//g;

    $s =~ s/ ,/,/g;
    $s =~ s/ 's/'s/g;

    return $s;
}
#>

#< END
END {
    LPost::destroy_tagger();
}
#>

1;
