#!/usr/bin/perl -w
#< Boilerplate
#
# The Aranea question answering system
# Copyright (C) 2002-2005, Jimmy Lin
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, please visit
# http://www.gnu.org/copyleft/gpl.html
#
# $Header: /fs/clip-qa/.cvsroot//Aranea/pm/Aranea/Lexicon/Lexicon.pm,v 1.20 2005/06/08 02:51:43 jimmylin Exp $
#
#>
#< POD
###############################################################################

=head1 NAME

B<C<Aranea::Lexicon::Lexicon>> - procedure for accessing lexicons (WordNet, CELEX, etc.)

=head1 SYNOPSIS

 use Aranea::Lexicon::Lexicon;
 
 my $root = stem('does');
 # result = [ [ 'do',  'v' ],
 #            [ 'doe', 'n' ] ]
 
 my $infl = inflect_verb('sing');
 # result =
 # [ ['a1S', 'sang', ''],
 #    'a2S', 'sang', ''],
 #    'a3S', 'sang', ''],
 #    'aP', 'sang', ''],
 #    'e1S', 'sing', '@'],
 #    'e2S', 'sing', '@'],
 #    'e3S', 'sings', '@+s'],
 #    'eP', 'sing', '@'],
 #    'i', 'sing', '@'],
 #    'pa', 'sung', ''],
 #    'pe', 'singing', '@+ing'] ]
 
 my $b1 = isPreposition('of');
 # result is true
 
 my $b2 = isStopword('the');
 # result is true
 
 my $b3 = isVerbInfinitive('test');
 # result is true

=head1 DESCRIPTION

=over 4

=item B<C<stem($word)>>

WordNet lemmatizer. Returns a ref to a table: the first column holds
the lemma, the second column its part of speech.  The rows are
arranged in ascending length of the lemma forms.

=item B<C<isPreposition($word)>>

Checks to see if the argument is a preposition.

=item B<C<isStopWord($word)>>

Checks to see if the argument is a stopword.  The stopwords list is
taken from WordNet 1.6

=item B<C<isVerbInfinitive($word)>>

Checks to see if the argument is a verb in its infinitive form.  This
list of verbs is taken from START.

=item B<C<inflect_verb($word)>>

Given a verb infinitive, returns its inflection table (conjugations).
The column of the table is the inflection form, the second column is
the inflected verb itself, and the third column describes how the
inflected form is composed.

The following cannocial inflection forms are provided (first column):
a1S, a2S, a3S, aP, e1S, e2S, e3S, eP, i, pa, pe.

And here is the key:
C<S> = Singular,
C<P> = Plural,
C<b> = positive,
C<c> = comparative,
C<s> = superlative,
C<i> = Infinitive,
C<p> = Participle,
C<e> = Present tense,
C<a> = Past tense,
C<1> = 1st person verb,
C<2> = 2nd person verb,
C<3> = 3rd person verb,
C<r> = Rare form,
C<X> = Headword form

=item B<C<get_idf>>

Returns the inverse document frequency of a token I<t> in the AQUAINT
corpus.  I<idf> = log(N/d), where I<N> is the total number of
documents in the corpus, and I<d> is the number of documents in the
corpus that contains the term I<t>.

=item B<C<get_log_tf>>

Returns the log term frequency of a token I<t> in the AQUAINT corpus,
i.e., log(c/W), where I<c> is the number of occurrences of token I<t>,
and I<W> is the total number of tokens in the corpus.

=back

=cut

###############################################################################
#>
#< Header
package Aranea::Lexicon::Lexicon;

use strict;
use warnings;

use Exporter;
use DB_File;

use Aranea::Common qw(find_file);

our @ISA = qw(Exporter);
our @EXPORT = qw();
our @EXPORT_OK = qw(is_geographic_location is_stopword is_preposition is_verb_infinitive inflect_verb stem get_idf get_tf get_log_tf get_WordNet_noundef);
our %EXPORT_TAGS = ( 'default' => [ qw(is_geographic_location is_stopword is_preposition is_verb_infinitive inflect_verb stem get_idf get_tf get_log_tf get_WordNet_noundef) ] );

our %TFCNT;          # unigram counts for log tf
our %IDFCNT;         # unigram doc counts for idf
our %CELEX_Verbs;    # CELEX verb inflection information
our %Stopword;
our %Preposition;

# regular inflection suffixes
our(@suffix, @suffixPOS, @suffix_keep);
# WordNet lemmas and exception information
our %WNSTEM;
our %WNEXCADJ;
our %WNEXCADV;
our %WNEXCN;
our %WNEXCV;

our %WordNetNounDefs;
#>

#< BEGIN

BEGIN {
    my $path = Aranea::Common::find_file("Aranea/Lexicon/stopwords.dat");
    my ($key, $value);

    # assume everything else is there in the same directory...
    $path =~ s/\/[^\/]+$//g;

    # Stopwords
    open(FILE, "< $path/stopwords.dat") or die "Stopwords not found!\n";
    chop, $Stopword{$_} = '1' while <FILE>;
    close(FILE);

    # Unigram counts for log term frequency
    dbmopen (%TFCNT, "$path/unigram-counts-AQUAINT.dbm", 0666) or
	die "Could not open unigram counts for calculating log tf!\n";

    # Unigram counts for inverse document frequency
    dbmopen (%IDFCNT, "$path/unigram-doc-counts-AQUAINT.dbm", 0666) or
	die "Could not open unigram doc counts for calculating idf!\n";

    # CELEX
    dbmopen (%CELEX_Verbs, "$path/celex_verb_infl.dbm", 0666) or
	die "Could not open CELEX inflections data file!\n";

    # WordNet Noun Definitions
    dbmopen (%WordNetNounDefs, "$path/wn_noun_dict.dbm", 0666) or
	die "Could not open WordNet definitions file!\n";

    #< Stemming info
    # words can end with these suffixes:  
    @suffix = ( # Noun suffixes
		"s","ses","xes","zes","ches","shes",
		# Verb suffixes 
		"s","ies","es","es","ed","ed","ing","ing",
		# Adjective suffixes 
		"er","est","er","est" );
    
    # the part of speech assigned to each of the above.
    @suffixPOS = ('n', 'n', 'n', 'n', 'n', 'n',
		  'v', 'v', 'v', 'v', 'v', 'v', 'v', 'v', 
		  'a', 'a', 'a', 'a' );

    # where this part should remain in the word:
    @suffix_keep = ( # Noun endings
		     "", "s", "x", "z", "ch", "sh", 
		     # Verb endings
		     "", "y", "e", "", "e", "", "e", "",
		     # Adjective endings
		     "", "", "e", "e" );

    # as in "suffixes" -> 3 -> "suffixes" - "xes" + "x" = "suffix";

    dbmopen %WNSTEM, "$path/wn_stem.dbm", 0666 or 
	die "Can't open DBM file\n";
    dbmopen %WNEXCADJ, "$path/wn_exc_adj.dbm", 0666 or 
	die "Can't open DBM file\n";
    dbmopen %WNEXCADV, "$path/wn_exc_adv.dbm", 0666 or 
	die "Can't open DBM file\n";
    dbmopen %WNEXCN, "$path/wn_exc_noun.dbm", 0666 or 
	die "Can't open DBM file\n";
    dbmopen %WNEXCV, "$path/wn_exc_verb.dbm", 0666 or 
	die "Can't open DBM file\n";
    #>
    #< The prepositions
    %Preposition = (
		    "about" => 1,
		    "above" => 1,
		    "across" => 1,
		    "after" => 1,
		    "against" => 1,
		    "along" => 1,
		    "among" => 1,
		    "around" => 1,
		    "at" => 1,
		    "before" => 1,
		    "behind" => 1,
		    "below" => 1,
		    "beneath" => 1,
		    "beside" => 1,
		    "between" => 1,
		    "by" => 1,
		    "down" => 1,
		    "during" => 1,
		    "except" => 1,
		    "for" => 1,
		    "from" => 1,
		    "in" => 1,
		    "inside" => 1,
		    "into" => 1,
		    "like" => 1,
		    "near" => 1,
		    "of" => 1,
		    "off" => 1,
		    "on" => 1,
		    "onto" => 1,
		    "over" => 1,
		    "past" => 1,
		    "since" => 1,
		    "through" => 1,
		    "to " => 1,
		    "toward" => 1,
		    "under " => 1,
		    "underneath " => 1,
		    "until " => 1,
		    "up" => 1,
		    "upon " => 1,
		    "with " => 1,
		    "within " => 1,
		    "without " => 1
		    );
    #>
}

#>

#< sub stem
sub stem {
    my $word = shift;
    my %posslemmas = ();

    if ( exists $WNSTEM{$word} ) {
	foreach my $t ( split //, $WNSTEM{$word} ) {
	    $posslemmas{$word}{$t} = 1;
	}

	#print STDERR "$word exists as '$WNSTEM{$word}'\n";
    }

    # Check standard endings
    for my $i (0 .. $#suffix ) {
	my $test = $word;

	if ( $test =~ s/$suffix[$i]$/$suffix_keep[$i]/ ) {

	    if ( exists $WNSTEM{$test} ) {
		my $possible_pos = $WNSTEM{$test};

		if ( $possible_pos =~ /$suffixPOS[$i]/ ) {
		    $posslemmas{$test}{$suffixPOS[$i]} = 1;

		    #print STDERR "$word ($test) exists as '$suffixPOS[$i]'\n";
		}
	    }
	}
    }


    $posslemmas{$WNEXCADJ{$word}}{'a'} = 1
	if exists $WNEXCADJ{$word};

    $posslemmas{$WNEXCADV{$word}}{'v'} = 1
	if exists $WNEXCADV{$word};

    $posslemmas{$WNEXCN{$word}}{'n'} = 1
	if exists $WNEXCN{$word};

    $posslemmas{$WNEXCV{$word}}{'v'} = 1
	if exists $WNEXCV{$word};

    my $results = [];
    foreach my $k ( sort { length($a) <=> length ($b) } keys %posslemmas ) {
	foreach my $ik ( sort keys %{$posslemmas{$k}} ) {
	    push @$results, [$k, $ik];
	}
    }

    if ( $#{@$results} == -1 and $word =~ /^[A-Z]/ ) {
	$word =~ tr/^[A-Z]/^[a-z]/;
	return stem($word);
    }

    return $results;
}
#>
#< sub is_preposition
sub is_preposition {
    my $s = shift;

    return unless $s;
    return ( exists($Preposition{$s}) or exists($Preposition{lc($s)}) );
}
#>
#< sub is_stopword
sub is_stopword {
    my $s = shift;

    return unless $s;
    return ( exists($Stopword{$s}) or exists($Stopword{lc($s)}) );
}
#>
#< sub is_verb_infinitive
sub is_verb_infinitive {
    my $s = shift;

    return unless $s;
    return ( exists($CELEX_Verbs{$s}) or exists($CELEX_Verbs{lc($s)}) );
}
#>
#< sub inflect_verb
sub inflect_verb {
    my $s = shift; 

    return unless $s;

    # get the CELEX entry
    my $txt = ( $CELEX_Verbs{$s} or $CELEX_Verbs{lc($s)} );
    return unless $txt;

    # convert CELEX entry into a table
    my @l = split("\n", $txt);
    my $r = [];
    map { push @$r, [split("\t", $_)]; } @l;

    return $r;
}
#>
#< sub get_idf
sub get_idf {
    my $w = shift;
    my $idf;

    # count summaries generated
    # by src/corpus-stats/generate-idf-hash.pl (v1.1)
    #   461348884 tokens read
    #     2074524 unique entries read
    #     1033461 total documents
    #  222.387826797858 average count
    #  122.646315010094 average doc count

    my $documents = 1033461;
    my $avg = 122.646315010094;

    if ( exists $IDFCNT{$w} ) {
	my $cnt = $IDFCNT{$w};
	$idf = log($documents/$cnt);
    } else {
	#print STDERR "Cannot calculate idf value for '$myword'\n";
	# do we really want to complain?
	$idf = log($documents/$avg);
    }

    return $idf;
}
#>
#< sub get_log_tf
sub get_log_tf {
    my $w = shift;
    my $logtf;

    my $total_tokens = 338665652;
    my $unique_tokens = 1720754;

    if ( exists $TFCNT{$w} ) {
	my $cnt = $TFCNT{$w};
	$logtf = log($cnt/$total_tokens);
    } else {
	#print STDERR "Cannot calculate idf value for '$myword'\n";
	# do we really want to complain?
	$logtf = log($unique_tokens/$total_tokens);
    }

    return $logtf;
}
#>
#< sub get_tf
sub get_tf {
    my $w = shift;
    my $tf;

    my $total_tokens = 338665652;
    my $unique_tokens = 1720754;

    if ( exists $TFCNT{$w} ) {
	my $cnt = $TFCNT{$w};
	$tf = $cnt/$total_tokens;
    } else {
	#print STDERR "Cannot calculate idf value for '$myword'\n";
	# do we really want to complain?
	$tf = $unique_tokens/$total_tokens;
    }

    return $tf;
}
#>
#< sub get_WordNet_noundef
sub get_WordNet_noundef {
    my $s = shift;

    $s =~ s/^the //;
    $s =~ s/ /_/g;

    return $WordNetNounDefs{$s};
}
#>

#< END
END {
    dbmclose %Aranea::Lexicon::Lexicon::TFCNT or
	die "Couldn't close DBM file!\n";

    dbmclose %Aranea::Lexicon::Lexicon::IDFCNT or
	die "Couldn't close DBM file!\n";

    dbmclose %Aranea::Lexicon::Lexicon::WNSTEM or
	die "Couldn't close DBM file!\n";

    dbmclose %Aranea::Lexicon::Lexicon::WNEXCADJ or
	die "Couldn't close DBM file!\n";

    dbmclose %Aranea::Lexicon::Lexicon::WNEXCADV or
	die "Couldn't close DBM file!\n";

    dbmclose %Aranea::Lexicon::Lexicon::WNEXCN or
	die "Couldn't close DBM file!\n";

    dbmclose %Aranea::Lexicon::Lexicon::WNEXCV or
	die "Couldn't close DBM file!\n";
}
#>

1;
