#!/usr/bin/perl -w
#< Boilerplate
#
# The Aranea question answering system
# Copyright (C) 2002-2005, Jimmy Lin
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, please visit
# http://www.gnu.org/copyleft/gpl.html
#
# $Header: /fs/clip-qa/.cvsroot//Aranea/pm/Aranea/Modules/GetSupport.pm,v 1.23 2005/06/10 16:17:16 jimmylin Exp $
#
#>
#< POD
###############################################################################

=head1 NAME

B<C<Aranea::Modules::GetSupport>> - Aranea module that finds
supporting documents for candidate answers

=head1 SYNOPSIS

This Aranea modules is meant to be used in conjunction with the
L<C<Aranea::Shell>|Aranea::Shell>.

=head1 DESCRIPTION

This Aranea modules finds supporting documents for each answer
candidate.  Only the URLs are recorded inside each
L<C<CandidateEntry>|Aranea::XML::CandidateEntry>; the page summaries
and other relevant information can be retrieved from the C<page_list>.
This module filters URLs with the same domain so that only one
supporting document is taken from each domain.

=cut

###############################################################################
#>
#< Header
package Aranea::Modules::GetSupport;

use strict;
use warnings;

use Data::Dumper;
use IO::Handle;

use Aranea::XML::Aranea;

#>

#< STDIN handler
unless ( caller() ) {
    my $stdin = new_from_fd IO::Handle(fileno(STDIN),"r");
    my $Aranea = Aranea::XML::Aranea->new($stdin);

    $Aranea = doModule($Aranea);

    $Aranea->write_xml();
}
#>

#< sub doModule
sub doModule {
    my $Aranea = shift;

    $Aranea->insert_pipeline_tag('$Id: GetSupport.pm,v 1.23 2005/06/10 16:17:16 jimmylin Exp $ ');

    #print Dumper($Aranea->candidate_list()->entries());

    # if there are no requests, then there is nothing to do
    return $Aranea if $Aranea->request_list()->entries_count() == -1;

    my $candidate_entries = $Aranea->candidate_list()->entries();
    my $page_list_entries = $Aranea->page_list()->entries();

    # sort by scores descending
    @$candidate_entries = sort { $b->score() <=> $a->score() }
      @$candidate_entries;

    my $i = 0;
    for my $candidate_entry (@$candidate_entries) {
	# Feb. 7, 2003 -- Recent profiling indicated that this module
	# is very very slow.  The fix is that we should only attempt
	# to find support for the top n candidates. (This also lead to
	# a tiny increase in performance.)
	last if $i > 100;
	$i++;

	my $candidate = $candidate_entry->candidate();
	# to keep track of duplicate URLs
	my %uniqueURLhash = ();

	for my $page_list_entry (@$page_list_entries) {
	    my $page_summary = $page_list_entry->page_summary();
	    my $url = $page_list_entry->page_url();

	    # get the base url (without path) use this to enforce
	    # uniqueness of supporting docs, i.e., we don't want more
	    # than one supporting doc from the same site.

	    my $url_sig = '';

	    # get the domain name
	    $url_sig = $& if $url =~ m|https?://[^/]+|;

	    # extract only the domain name, e.g., collapse
	    # http://var1.foo.com/ and http://var2.foo.com/ into the
	    # same signature
	    $url_sig = $1 if $url_sig =~ m|//\w+\.(\w+.\w+)$|;
	    next unless $url_sig;

	    # search for support (a match either with or without
	    # commas), but make sure it's an unique URL

	    # Feb. 3, 2003 -- make sure that these candidates aren't
	    # embedded inside other tokens, i.e., they are bracketed
	    # by spaces or punctuation.

	    if ( $page_summary =~ /(\W|^)\Q$candidate\E(\W|$)/i and
		 not exists($uniqueURLhash{$url_sig}) ) {
	      $candidate_entry->add_support($url);
	      $uniqueURLhash{$url_sig} = 1;
	    }
	}
    }

    # discard candidate answers that have support from two or less
    # unique pages.  If we can't confirm the answer with multiple
    # sources, then we should trust it anyway.

    # Feb 4, 2003 -- "discard with two support" seems to give better
    # performance than "discard with one support". I keep tweaking and
    # re-discovering this...

    my $new_entries = discard_with_two_support($candidate_entries);
    $Aranea->candidate_list()->entries($new_entries);

    #print Dumper($Aranea->candidate_list()->entries());

    return $Aranea;
}
#>

#< discard_with_two_support
sub discard_with_two_support {
    my $candidate_entries = shift;
    my $new_entries = [];

    # rewrite this using 'filter_entries' method of CandidateList
    for my $candidate_entry (@$candidate_entries) {
	push @$new_entries, $candidate_entry
	    if $candidate_entry->support_count() > 1;
    }

    return $new_entries;
}
#>

1;
