#!/usr/bin/perl -w
#< Boilerplate
#
# The Aranea question answering system
# Copyright (C) 2002-2005, Jimmy Lin
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, please visit
# http://www.gnu.org/copyleft/gpl.html
#
# $Header: /fs/clip-qa/.cvsroot//Aranea/pm/Aranea/Modules/LookupDate.pm,v 1.9 2005/06/10 16:17:16 jimmylin Exp $
#
#>
#< POD
###############################################################################

=head1 NAME

B<C<Aranea::Modules::LookupDate>> - Aranea module for answering date
questions using database techniques

=head1 SYNOPSIS

This Aranea modules is meant to be used in conjunction with the
L<C<Aranea::Shell>|Aranea::Shell>.

=head1 DESCRIPTION

This module answers the following types of date questions:

=over 4

=item *

The birth date, death date, and life span (e.g., 1879-1955) of a
person. (from Biography.com)

=item *

The date a state was admitted to the Union. (from 50States)

=back

=cut

###############################################################################
#>
#< Header
package Aranea::Modules::LookupDate;

use strict;
use warnings;

use IO::Handle;

use Aranea::XML::Aranea;
use Aranea::Resources::Gateway;

#>

#< STDIN handler
unless ( caller() ) {
    my $stdin = new_from_fd IO::Handle(fileno(STDIN),"r");
    my $Aranea = Aranea::XML::Aranea->new($stdin);

    $Aranea = doModule($Aranea);

    $Aranea->write_xml();
}
#>

#< sub doModule
sub doModule {
    my $Aranea = shift;

    $Aranea->insert_pipeline_tag('$Id: LookupDate.pm,v 1.9 2005/06/10 16:17:16 jimmylin Exp $ ');

    my $query = $Aranea->query()->text();

    # the birthdate of a person from biography.com
    db_insert($Aranea, "Biography", [get_entity($query), "birthdate"])
	if birthdate_sig($query);

    # the deathdate of a person from biography.com
    db_insert($Aranea, "Biography", [get_entity($query), "deathdate"])
	if deathdate_sig($query);

    # the lifespan of a person from biography.com
    db_insert($Aranea, "Biography", [get_entity($query), "lifespan"])
	if lifespan_sig($query);

    # the statehood data from 50 states
    db_insert($Aranea, "50states", [get_entity($query), "statehood"])
	if statehood_sig($query);

    return $Aranea;
}
#>

#< sub birthdate_sig
sub birthdate_sig {
    my $query = shift;
    $query =~ /(born|birthday|birthdate|birth date)/i and
	$query =~ /(when|day|month|year)/i;
}
#>
#< sub deathdate_sig
sub deathdate_sig {
    my $query = shift;
    $query =~ /(die|death)/i and
	$query =~ /(when|day|month|year)/i and
	    $query !~ /^How old/i;
}
#>
#< sub lifespan_sig
sub lifespan_sig {
    my $query = shift;
    $query =~ /did.*live\??$/i and
	$query =~ /(when|century|year)/i;
}
#>
#< sub statehood_sig
sub statehood_sig {
    my $query = shift;
    $query =~ /become a state|(enter|join) the union/i;
}
#>

#< sub get_entity
sub get_entity {
    my $query = shift;
    $query =~ s/^[A-Z][^ ]+ //;

    if ( $query =~ /[A-Z][^ ]+( [A-Z][^ ]+)*/ ) {
	my $s = $&;
	$s =~ s/'s$//g;          #'//g;
	return $s;
    }

    return '';
}
#>

#< sub insert_answers
sub insert_answers {
    my ($Aranea, $source, $results) = @_;

    foreach my $ans ( @$results ) {
	my $entry = Aranea::XML::CandidateEntry->new($ans, 999999);
	$entry->add_support($source);
	$Aranea->candidate_list()->add_entry($entry);
    }
}
#>

1;
