#!/usr/bin/perl -w
#< Boilerplate
#
# The Aranea question answering system
# Copyright (C) 2002-2005, Jimmy Lin
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, please visit
# http://www.gnu.org/copyleft/gpl.html
#
# $Header: /fs/clip-qa/.cvsroot//Aranea/pm/Aranea/Modules/LookupLocation.pm,v 1.12 2005/06/10 16:17:16 jimmylin Exp $
#
#>
#< POD
###############################################################################

=head1 NAME

B<C<Aranea::Modules::LookupLocation>> - Aranea module for answering
location questions using database techniques

=head1 SYNOPSIS

This Aranea modules is meant to be used in conjunction with the
L<C<Aranea::Shell>|Aranea::Shell>.

=head1 DESCRIPTION

This module answers the following types of location questions:

=over 4

=item *

The location of a place, i.e., Where is X? (from WordNet)

=item *

The birthplace of a person. (from Biography.com)

=back

=cut
###############################################################################
#>
#< Header
package Aranea::Modules::LookupLocation;

use strict;
use warnings;

use Aranea::XML::Aranea;
use Aranea::XML::CandidateEntry;
use Aranea::Common qw(:default);
use Aranea::Lexicon::Lexicon qw(:default);
use Aranea::Patterns::Locations qw(exactify_location);
use Aranea::Resources::Gateway;

#>

#< STDIN handler
unless ( caller() ) {
    my $stdin = new_from_fd IO::Handle(fileno(STDIN),"r");
    my $Aranea = Aranea::XML::Aranea->new($stdin);

    $Aranea = doModule($Aranea);

    $Aranea->write_xml();
}
#>

#< sub doModule
sub doModule {
    my $Aranea = shift;

    $Aranea->insert_pipeline_tag('$Id: LookupLocation.pm,v 1.12 2005/06/10 16:17:16 jimmylin Exp $ ');

    my $query = $Aranea->query()->text();
    my $entity;

    # the location of an entity from WordNet
    if ( $entity = location_sig($query) ) {
	if ( my $def = extract_locative(get_WordNet_noundef($entity)) ) {
	    my $entry = Aranea::XML::CandidateEntry->new($def, 999999);
	    $entry->add_support("WordNet17");
	    $Aranea->candidate_list()->add_entry($entry);
	}
    }

    # the birthplace of a person from biography.com
    db_insert($Aranea, "Biography", [$entity, "birthplace"])
	if $entity = birthplace_sig($query);

    return $Aranea;
}
#>

#< sub location_sig
sub location_sig {
    my $s = shift;
    my $entity = '';

    $entity = $1 if $s =~ /Where is (.*) located\?/i;
    $entity = $1 if $s =~ /Where is (.*)\?/ and not $entity;

    return $entity;
}
#>
#< sub birthplace_sig
sub birthplace_sig {
    my $query = shift;
    return ($query =~ /(born|birth\s?place)/i and 
	    $query =~ /where/i) ? get_entity($query) : '';
}
#>

#< sub extract_locative
sub extract_locative {
    my $def = shift;
    return unless $def;

    my $tagged = pos_tag($def);
    my $ans = '';

    # from the WordNet definitions, search for a locative PP
    if ( $tagged =~ /(in|on)\/IN( [^\/]+\/(DT|JJ))*( ?[^\/]+\/(NN|IN|NNP|NNPS))+/ ) {
	$ans = $&;
	$ans =~ s/ [^\/]+\/IN\s*$//g;  # if the ending word get mistagged as a preposition
	$ans =~ s/\/[^ ]+//g;
    }

    # otherwise, take the first proper named entity as the answer

    # Note, this heuristic screws up sometimes. Take, for example, the
    # definition of Eiffel Tower -- 'a wrought iron tower 300 meters
    # high that was constructed for the Paris Exhibition of 1889; for
    # many years it was the tallest man-made structure'.  The first NP
    # is 'Paris Exhibition', which makes for a bad answer (inexact).
    # Looking over the TREC9-10 data, however, we win more often than
    # we lose -- jimmylin, July 25, 2003

    unless ( $ans ) {
	$def =~ /[A-Z][^ ]+( [A-Z][^ ]+)*/;
	$ans = $&;
    }

    $ans =~ s/[;,.]+$//g;

    # if the answer doesn't have caps, then something is wrong...
    return '' unless $ans =~ /[A-Z]/;

    return exactify_location($ans);
}

#>
#< sub get_entity
sub get_entity {
    my $query = shift;
    $query =~ s/^[A-Z][^ ]+ //;

    return $query =~ /[A-Z][^ ]+( [A-Z][^ ]+)*/ ? $& : '';
}
#>
#< sub insert_answers
sub insert_answers {
    my ($Aranea, $source, $results) = @_;

    foreach my $ans ( @$results ) {
	my $entry = Aranea::XML::CandidateEntry->new($ans, 999999);
	$entry->add_support($source);
	$Aranea->candidate_list()->add_entry($entry);
    }
}
#>

1;
