#!/usr/bin/perl -w
#< Boilerplate
#
# The Aranea question answering system
# Copyright (C) 2002-2005, Jimmy Lin
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, please visit
# http://www.gnu.org/copyleft/gpl.html
#
# $Header: /fs/clip-qa/.cvsroot//Aranea/pm/Aranea/Shell.pm,v 1.14 2005/06/08 02:51:42 jimmylin Exp $
#
#>
#< POD
###############################################################################

=head1 NAME

B<C<Aranea::Shell>> - the XML Shell for connecting Aranea
modules together

=head1 SYNOPSIS

  my $modules = [ "CreateObject", ... "ConvertToTREC" ];
  my $args = ["Where is Belize located", 202];
 
  my $shell = Aranea::Shell->new(MODULES => $modules,
                                 ARGS => $args);
  $shell->load_modules();
 
  my $Aranea = $shell->execute();

=head1 DESCRIPTION

This Perl module provides the glue that binds the Aranea modules
together, through the shared B<C<Aranea::XML::Aranea>> object.
Because all Aranea modules have the exact same input/output behavior,
it is possible to automate the process of running a series of modules
in sequence.  The C<Aranea::Shell> object provides the necessary
infrastructure and support for such a task.

=head1 METHODS

=over 4

=cut

###############################################################################
#>
#< Header
package Aranea::Shell;

use strict;
use warnings;

use Aranea::XML::Aranea;
use Aranea::Common qw(:default);

our $AUTOLOAD;

#>

#< defined fields
my %fields = (
	      MODULES => undef,
	      ARGS => undef,
	      FILE => undef,
	      );
#>
#< sub new

=item B<C<new(named ARGS)>>

Constructs a new Aranea Shell.  The constructor takes (optional) named
arguments:

=over 4

=item B<C<MODULES>>

Listref to modules to execute.  The shell assumes that the fully
qualified name of the module is B<C<Aranea::Modules::MyModule>>.

=item B<C<ARGS>>

Listref to arguments that should be passed into the first Aranea
module.  The first module that the shell executes must be treated
differently from the rest (i.e., some part of the program must be
responsible for creating the initial XML object or reading it in from
file/stdout).  If the C<ARGS> argument is specified, the shell
deferences the listref and passes the list as the arguments of the
first module.  For example, the module C<CreateObject> generates an
empty Aranea XML object with only the question number and the question
text:

  my $modules = [ "CreateObject", ... ];
  my $args = [ "where is Belize located?", 202 ];
  my $shell = Aranea::Shell->new(MODULES => $modules,
                                 ARGS => $args);

=item B<C<FILE>>

Scalar value specifying the filename containing the Aranea XML object
to be read.  The specified XML file will read and parsed into an
Aranea XML object.  Here is a sample call:

  my $shell = Aranea::Shell->new(MODULES => $modules,
                                 FILE => "foo.xml");

=back

Optionally, one can construct an empty Aranea Shell and then populate
the parameters using corresponding mutator methods.

=cut
sub new {
    my ($proto, %args) = @_;
    my $class = ref($proto) || $proto;

    # taking the reference to the named arguments as the object
    # itself
    my $self = \%args;
    $self->{_permitted} = \%fields;

    bless ($self, $class);
    return $self;
}
#>
#< sub AUTOLOAD

=item B<C<MODULES([$listref])>>

Accesses or mutates the listref of modules to run with this Aranea
Shell.

=item B<C<ARGS([$listref])>>

Accesses or mutates the listref or parameters to pass into the first
module of the Aranea Shell.

=item B<C<FILE([$filename])>>

Accesses or mutates the name of tile XML file that should serve as
input to the first module of the Aranea Shell.

=cut
sub AUTOLOAD {
    my $self = shift;
    my $type = ref($self)
	or die "$self is not an object";

    my $name = $AUTOLOAD;
    $name =~ s/.*://;   # strip fully-qualified portion
    if ($name =~ m/DESTROY/) {
	return;
    }

    unless (exists $self->{_permitted}->{$name} ) {
	die "Can't access `$name' field in class $type";
    }

    if (@_) {
	return $self->{$name} = shift;
    }
    else {
	return $self->{$name};
    }
}
#>

#< sub load_modules

=item B<C<load_modules()>>

Loads the modules indicated in this Aranea Shell object.  Note that
this method only needs to invoked once per instance of the Aranea
Shell.

=cut
sub load_modules {
    my $self = shift;
    my $modules = $self->{MODULES};

    # Load all the modules first; no sense in actually running the
    # modules if there are errors

    foreach my $m ( @$modules ) {
	my $path = find_file("Aranea/Modules/${m}.pm")
	    or die "Fatal Error: couldn't find module $m.\n";
	require $path or
	    die "Fatal Error: Couldn't load module $m.\n";
    }
}
#>
#< sub execute

=item B<C<execute()>>

Runs the Aranea shell.  This method assumes proper setup: all modules
have been loaded, and a valid method exists for starting the pipeline.

=cut
sub execute {
    my $self = shift;
    my $modules = $self->{MODULES};

    my $Aranea;
    my $pipe_start = '';

    # determine how to start the first module
    if ( exists $self->{ARGS} ) {
	# the first module should be treated specially
	$pipe_start = shift @$modules;

	# NOTES: the above 'shift' is destructive and mutates
	# @$modules.  However, I'm careful to restore it later - this
	# makes subsequent code much more elegant.

	{
	    no strict "refs";
	    # pass in command line parameters
	    #&{$callback}($pipe_start) if $callback;
	    my $param = $self->{ARGS};
	    $Aranea = &{"Aranea::Modules::${pipe_start}::doModule"}(@$param);
	}
    }
    elsif ( exists $self->{FILE} ) {
	# create the Aranea object directly from XML
	my $param = $self->{FILE};
	$Aranea = Aranea::XML::Aranea->new($param);
    }
    else {
	die "Fatal Error: Unknown method for starting the pipeline.\n";
    }

    # pipe the output through all the other modules
    foreach my $m ( @$modules ) {
	{
	    no strict "refs";
	    #&{$callback}($m) if $callback;

	    if ( exists $self->{INEXACT_SNIPPETS_COUNT} ) {
		Aranea::Modules::GenerateBackoffRequest::set_inexact_snippets_count($self->{INEXACT_SNIPPETS_COUNT}) if $m eq 'GenerateBackoffRequest';

		Aranea::Modules::GenerateReformulations::set_inexact_snippets_count($self->{INEXACT_SNIPPETS_COUNT}) if $m eq 'GenerateReformulations';
	    }

	    if ( exists $self->{EXACT_SNIPPETS_COUNT} ) {
		Aranea::Modules::GenerateReformulations::set_exact_snippets_count($self->{EXACT_SNIPPETS_COUNT}) if $m eq 'GenerateReformulations';
	    }

	    $Aranea = &{"Aranea::Modules::${m}::doModule"}($Aranea);
	}
    }

    # restore the first module in the pipeline
    unshift @$modules, $pipe_start if $pipe_start;

    return $Aranea;
}
#>

1;
