#!/usr/bin/perl -w
#< Boilerplate
#
# The Aranea question answering system
# Copyright (C) 2002-2005, Jimmy Lin
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, please visit
# http://www.gnu.org/copyleft/gpl.html
#
# $Header: /fs/clip-qa/.cvsroot//Aranea/pm/Aranea/XML/Aranea.pm,v 1.41 2005/06/08 01:41:46 jimmylin Exp $
#
#>
#< POD
###############################################################################

=head1 NAME

B<C<Aranea::XML::Aranea>> - object representing the Aranea XML object

=head1 SYNOPSIS

  use Aranea::XML::Aranea;
 
  my $stdin = new_from_fd IO::Handle(STDIN, "r");
  my $Aranea = Aranea::XML::Aranea->new($stdin);
 
  my $request_list = $Aranea->request_list();
  my $page_list = $Aranea->page_list();
  my $candidate_list = $Aranea->candidate_list();
 
  $Aranea->write_xml();   # prints the XML (optional arg = filename)

=head1 DESCRIPTION

Given an XML input stream, this object parses the input stream into a
data structure, and knows how to write the data structure out.

=head1 METHODS

=over 4

=cut

###############################################################################
#>
#< Header
package Aranea::XML::Aranea;

use strict;
use warnings;

use Date::Format;
use Data::Dumper;
use IO::File;
use XML::Simple;
use XML::Writer;

use Aranea::XML::Query;
use Aranea::XML::RequestList;
use Aranea::XML::CandidateList;
use Aranea::XML::PageList;

our $AUTOLOAD;

#>

#< defined fields
my %fields = (
	      query => undef,
	      candidate_list => undef,
	      page_list => undef,
	      request_list => undef,
	      );
#>
#< sub new

=item B<C<new([$filename])>>

Creates a new Aranea XML object from C<$filename>.  To read XML from
C<stdin>:

  my $stdin = new_from_fd IO::Handle(STDIN, "r");
  my $Aranea = Aranea::XML::Aranea->new($stdin);

=cut
sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $handle = shift;
    my $data = {};

    if ( $handle ) {
	$data = XMLin($handle,
		      'forcearray' => ['entry','limit','module','doc'],
		      'keyattr' => []);
    }

    my $self = {
		_permitted => \%fields,
		%fields
	       };

    # automatically create objects correponding to the major XML sections
    $self->{request_list} =
	Aranea::XML::RequestList->new($data->{request_list});
    $self->{page_list} =
	Aranea::XML::PageList->new($data->{page_list});
    $self->{candidate_list} =
	Aranea::XML::CandidateList->new($data->{candidate_list});
    $self->{query} =
	Aranea::XML::Query->new($data->{query});

    if ($data->{pipeline}->{module}) {
	$self->{pipeline} = $data->{pipeline}->{module};
    }

    bless ($self, $class);
    return $self;
}
#>
#< sub AUTOLOAD

=item B<C<request_list([$request_list])>>

Accesses or mutates the L<B<C<RequestList>>|Aranea::XML::RequestList>
object, which corresponds the B<C<E<lt>request_listE<gt>>>) or the
Aranea XML object.

=item B<C<page_list([$page_list])>>

Accesses or mutates the L<B<C<PageList>>|Aranea::XML::PageList>
object, which corresponds the B<C<E<lt>page_listE<gt>>>) or the
Aranea XML object.

=item B<C<candidate_list([$candidate_list])>>

Accesses or mutates the
L<B<C<CandidateList>>|Aranea::XML::CandidateList> object, which
corresponds the B<C<E<lt>candidate_listE<gt>>>) or the Aranea XML
object.

=cut
sub AUTOLOAD {
    my $self = shift;
    my $type = ref($self)
	or die "$self is not an object";

    my $name = $AUTOLOAD;
    $name =~ s/.*://;   # strip fully-qualified portion
    if ($name =~ m/DESTROY/) {
	return;
    }

    unless (exists $self->{_permitted}->{$name} ) {
	die "Can't access `$name' field in class $type";
    }

    if (@_) {
	return $self->{$name} = shift;
    }
    else {
	return $self->{$name};
    }
}
#>

#< sub insert_pipeline_tag

=item B<C<insert_pipeline_tag($id)>>

Appends a timestamped identifier to the pipeline section of the Aranea
XML object.  The identifier is usually the CVS C<ID> keyword from the
calling program.

=cut
sub insert_pipeline_tag {
    my $self = shift;
    my $name = shift;
	my $other_info = shift;

    # if the ID passed in was a CVS expanded keyword, we want to extract
    # the relevant info from the CVS keyword.  Otherwise, if we saved
    # the results to a file and checked the file into CVS, the ID would
    # get replaced with the save file info... thereby erasing the
    # pipeline info.
    if ( $name =~ /\$(Id:) ([^ ]+ \d+\.\d+) .*\$/ ) {

	# the above regexp has the tendency to be recognized as a cvs
	# keyword upon check-in, hence screwing up the regexp.  the
	# parens should fix it.  damn cvs for not providing a feature to
	# override keyword substitution.
	$name = $2;
    }

    if (! $name) {
	die "You must pass a description of the module creating this object.  Usually \' \$Id: Aranea.pm,v 1.41 2005/06/08 01:41:46 jimmylin Exp $ \' (the CVS tag) is the right thing.\n";
    }

    my $pipeline_entry = {};
    $pipeline_entry->{'name'} = $name;
    my @lt = localtime(time);
    $pipeline_entry->{'date'} = strftime("%c",@lt);

	if ($other_info) {
		$pipeline_entry->{'other_info'} = $other_info;
	}

    push @{$self->{pipeline}}, $pipeline_entry;
}
#>
#< sub write_xml

=item B<C<write_xml([$filename])>>

Prints out the XML of the Aranea XML object to C<STDOUT>.  If passed
an optional arugment, the XML will be written to a file.

=cut
sub write_xml {
    my $self = shift;
    my $outfile = shift;
    my $writer;

    my $output;
    if ( $outfile ) {
	$output = new IO::File("> $outfile");
	$writer = new XML::Writer(DATA_INDENT=>5,
				  DATA_MODE => 'true',
				  OUTPUT => $output);
    }
    else {
	$writer = new XML::Writer(DATA_INDENT=>5, DATA_MODE => 'true');
    }

    # prints the xml declaration specifying the encoding format:
    #   <?xml version="1.0" encoding="ISO-8859-1"?>
    # without this, some XML parsers will have problems parsing
    $writer->xmlDecl("ISO-8859-1");

    $writer->startTag('aranea');

    # dispatch to the XML writer of each object
    $self->query()->write_xml($writer);
    $self->request_list()->write_xml($writer);
    $self->page_list ()->write_xml($writer);
    $self->_print_pipeline($writer);
    $self->candidate_list()->write_xml($writer);

    $writer->endTag();
    $writer->end();

    $output->close() if $outfile;
}
#>
#< sub _print_pipeline
# This is a private method that writes an XML representation of the
# module pipeline.
sub _print_pipeline {
    my $self = shift;
    my $writer = shift;
    my $pipeline = $self->{pipeline};

    $writer->startTag('pipeline');

    for my $entry (@$pipeline) {
	$writer->startTag('module');
	my @pipeline_keys = keys(%$entry);

	for my $pipeline_key (@pipeline_keys) {

	    if ($pipeline_key eq 'other_info') {
		$writer->startTag($pipeline_key);
		my $other_hash = $entry->{$pipeline_key};
		for my $other_key (keys(%$other_hash)) {
		    $writer->startTag($other_key);
		    $writer->characters($other_hash->{$other_key});
		    $writer->endTag();
		}
		$writer->endTag();
	    }
	    else {
		$writer->startTag($pipeline_key);
		$writer->characters($entry->{$pipeline_key});
		$writer->endTag();
	    }
	}
	$writer->endTag();
    }
    $writer->endTag('pipeline');
}
#>

#< More POD
###############################################################################

=back

=head1 HINTS

=over 4

=item B<How do I get access to the question?>

Something like this will do:

  my $q = $Aranea->query()->text();
  my $qid = $Aranea->query()->id();

=back

=cut

###############################################################################
#>

1;
