#!/usr/bin/perl -w
#< Boilerplate
#
# The Aranea question answering system
# Copyright (C) 2002-2005, Jimmy Lin
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, please visit
# http://www.gnu.org/copyleft/gpl.html
#
# $Header: /fs/clip-qa/.cvsroot//Aranea/pm/Aranea/XML/PageEntry.pm,v 1.8 2005/06/08 01:41:47 jimmylin Exp $
#
#>
#< POD
###############################################################################

=head1 NAME

B<C<Aranea::XML::PageEntry>> - object representing the B<C<E<lt>page_entryE<gt>>> of an Aranea XML object

=head1 SYNOPSIS

  my $page_list = $Aranea->candidate_list();
  my $entries = $page_list->entries();
 
  for my $entry ( @$entries ) {
     # do stuff...
  }

=head1 DESCRIPTION

The C<PageEntry> object represents a single entry inside the
C<page_list> section of an Aranea XML object.  Here is an example:

  <entry>
    <id>0</id>
    <page_summary>here is some text</page_summary>
    <page_cache_url> http://www.foo.com/cache.html </page_cache_url>
    <page_title>Sample</page_title>
    <page_paragraph>This is a pragraph and here is some text in that paragraph that was returned as a summary.</page_paragraph>
    <page_url> http://www.foo.com/sample.html </page_url>
  </entry>

The L<C<PageList>|Aranea::XML::PageList> object serves as a container
for a collection of C<PageEntry>s.

=head1 METHODS

=over 4

=cut

###############################################################################
#>
#< Header
package Aranea::XML::PageEntry;

use strict;
use warnings;

use XML::Simple;
use XML::Writer;

our $AUTOLOAD;

#>

#< defined fields
my %fields = (
	      id => undef,
	      page_url => undef,
	      page_title => undef,
	      page_category => undef,
	      page_summary => undef,
	      page_url_cache => undef,
	      page_paragraph => undef,
	     );
#>
#< sub new

=item B<C<new()>>

Constructs an empty C<PageEntry>.

=cut

sub new {
  my $proto = shift;
  my $data = shift;
  my $class = ref($proto) || $proto;
  my $self;

  if ( $data ) {
    $self = $data;
    $self->{_permitted} = \%fields;
  } else {
    $self = {
	     _permitted => \%fields,
	     %fields
	    };
  }

  bless ($self, $class);
  return $self;
}
#>
#< sub AUTOLOAD

=item B<C<id([$scalar])>>

Accesses or mutates the id.  This value identifies the request from
which this C<PageEntry> derives.

=item B<C<page_url([$scalar])>>

Accesses or mutates the page URL.

=item B<C<page_title([$scalar])>>

Accesses or mutates the page title.

=item B<C<page_summary([$scalar])>>

Accesses or mutates the page summary.

=item B<C<page_paragraph([$scalar])>>

Accesses or mutates the page paragraph.

=item B<C<page_category([$scalar])>>

Accesses or mutates the page category.  Google cross indexes its
search results with Yahoo categories.

=item B<C<page_url_cache([$scalar])>>

Accesses or mutates the URL of the cached page.

=cut
sub AUTOLOAD {
    my $self = shift;
    my $type = ref($self)
	or die "$self is not an object";

    my $name = $AUTOLOAD;
    $name =~ s/.*://;   # strip fully-qualified portion
    if ($name =~ m/DESTROY/) {
	return;
    }

    unless (exists $self->{_permitted}->{$name} ) {
	die "Can't access `$name' field in class $type";
    }

    if (@_) {
	return $self->{$name} = shift;
    }
    else {
	return $self->{$name};
    }
}
#>

#< sub write_xml

=item B<C<write_xml($writer)>>

Writes the XML representation of this C<CandidateList>; takes an
C<XML::Writer> as its only argument.  Note that this method is rarely
invoked manually; under typical usage patterns, the C<write_xml>
method of L<C<Aranea::XML::Aranea>|Aranea::XML::Aranea> will call this
method automatically.

=cut
sub write_xml {
    my $self = shift;
    my $writer = shift;

    $writer->startTag('entry');
    for my $key ( sort keys %fields ) {
	next unless defined $self->{$key};

	$writer->startTag($key);
	$writer->characters($self->{$key});
	$writer->endTag();
    }
    $writer->endTag('entry');
}
#>

1;
