#!/usr/bin/perl -w
#< Boilerplate
#
# The Aranea question answering system
# Copyright (C) 2002-2005, Jimmy Lin
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, please visit
# http://www.gnu.org/copyleft/gpl.html
#
# $Header: /fs/clip-qa/.cvsroot//Aranea/pm/Aranea/XML/PageList.pm,v 1.15 2005/06/08 01:41:47 jimmylin Exp $
#
#>
#< POD
###############################################################################

=head1 NAME

B<C<Aranea::XML::PageList>> - object representing the B<C<E<lt>page_listE<gt>>> of an Aranea XML object

=head1 SYNOPSIS

  use Aranea::XML::Aranea;
  use Aranea::XML::PageList;
 
  my $stdin = new_from_fd IO::Handle(STDIN, "r");
  my $Aranea = Aranea::XML::Aranea->new($stdin);
 
  my $page_list = $Aranea->page_list();
  my $entries = $page_list->entries();  # arrayref to entries in page list

=head1 DESCRIPTION

The C<PageList> object represents the C<page_list> section of an
Aranea XML object.  Here is an example:

  <page_list>
    <entry>
      <id>0</id>
      <page_summary>here is some text</page_summary>
      <page_cache_url> http://www.foo.com/cache.html </page_cache_url>
      <page_title>Sample</page_title>
      <page_url> http://www.foo.com/sample.html </page_url>
    </entry>
    ...
  </page_list>

The C<PageList> object serves as a container for a collection of
L<C<PageEntry>s|Aranea::XML::PageEntry>.  Under normal circumstances,
the creation of a C<PageList> should be handled automatically by
the L<C<Aranea::XML::Aranea>|Aranea::XML::Aranea> object.

=head1 METHODS

=over 4

=cut

###############################################################################
#>
#< Header
package Aranea::XML::PageList;

use strict;
use warnings;

use Data::Dumper;
use XML::Simple;
use XML::Writer;

use Aranea::XML::PageEntry;

#>

#< sub new

=item B<C<new($data)>>

Constructs a C<PageList> object given parsed XML data.  Note that this
method is rarely invoked manually; under typical usage patterns, the
constructor of L<C<Aranea::XML::Aranea>|Aranea::XML::Aranea> will call
this method automatically.

=cut
sub new {
  my $proto = shift;
  my $data = shift;
  my $class = ref($proto) || $proto;
  my $self = $data;

  if ( $self->{entry} ) {
    map { Aranea::XML::PageEntry->new($_) } @{$self->{entry}};
  } else {
    $self->{entry} = [];
  }

  #print Dumper($self);

  bless ($self, $class);
  return $self;
}
#>

#< sub entries

=item B<C<entries([$listref])>>

Accesses or mutates the collection of
L<C<PageEntry>s|Aranea::XML::PageEntry>.  When called with no
arguments, this method returns a listref of C<PageEntry>s.  When
called with a single listref, the collection of C<PageEntry>s is set
to the listref.

The typical usage pattern is to iterate over each page object and perform
some action, e.g.,

  my $page_list = $Aranea->page_list();
  my $entries = $page_list->entries();
 
  for my $entry ( @$entries ) {
      # do stuff...
  }

=cut
sub entries {
    my $self = shift;
    return $self->{entry};
}
#>
#< sub add_entry

=item B<C<add_entry($entry)>>

Adds a L<C<PageEntry>|Aranea::XML::PageEntry> to this C<PageList>.

=cut
sub add_entry {
    my $self = shift;
    my $entry = shift;

    push @{$self->entries()}, $entry;
}

#>
#< sub write_xml

=item B<C<write_xml($writer)>>

Writes the XML representation of this C<PageList>; takes an
C<XML::Writer> as its only argument.  Note that this method is rarely
invoked manually; under typical usage patterns, the C<write_xml>
method of L<C<Aranea::XML::Aranea>|Aranea::XML::Aranea> will call this
method automatically.

=cut
sub write_xml {
    my $self = shift;
    my $writer = shift;

    $writer->startTag('page_list');
    map { $_->write_xml($writer) } @{$self->entries()};
    $writer->endTag('page_list');
}
#>

1;
