#!/usr/bin/perl -w
#< Boilerplate
#
# The Aranea question answering system
# Copyright (C) 2002-2005, Jimmy Lin
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, please visit
# http://www.gnu.org/copyleft/gpl.html
#
# $Header: /fs/clip-qa/.cvsroot//Aranea/pm/Aranea/XML/RequestList.pm,v 1.19 2005/06/08 01:41:47 jimmylin Exp $
#
#>
#< POD
###############################################################################

=head1 NAME

B<C<Aranea::XML::RequestList>> - object representing the B<C<E<lt>request_listE<gt>>> of an Aranea XML object

=head1 SYNOPSIS

  use Aranea::XML::Aranea;
  use Aranea::XML::RequestList;
 
  my $stdin = new_from_fd IO::Handle(STDIN, "r");
  my $Aranea = Aranea::XML::Aranea->new($stdin);
 
  my $request_list = $Aranea->request_list();
  my $entries = $request_list->entries();
  # arrayref to entries in request list

=head1 DESCRIPTION

The C<RequestList> object represents the C<request_list> section of an
Aranea XML object.  Here is an example:

  <request_list>
    <entry>
      <query>What is the volcano that destroyed the ancient city of Pompeii?</query>
      <score>1</score>
      <constraints>
        <use_backoff>true</use_backoff>
        <pages_to_search>100</pages_to_search>
      </constraints>
    </entry>
    <entry>
      <query>+the +volcano +that +destroyed +the +ancient +city +of +Pompeii</query>
      <score>1</score>
      <constraints>
        <use_backoff>true</use_backoff>
        <pages_to_search>100</pages_to_search>
      </constraints>
    </entry>
  </request_list>

The C<RequestList> object serves as a container for a collection of
L<C<RequestEntry>s|Aranea::XML::RequestEntry>.  Under normal
circumstances, the creation of a C<RequestList> should be handled
automatically by the L<C<Aranea::XML::Aranea>|Aranea::XML::Aranea>
object.

=head1 METHODS

=over 4

=cut

###############################################################################
#>
#< Header
package Aranea::XML::RequestList;

use strict;
use warnings;

use Data::Dumper;
use XML::Simple;
use XML::Writer;

use Aranea::XML::RequestEntry;

#>

#< sub new

=item B<C<new($data)>>

Constructs a C<RequestList> object given parsed XML data.  Note that
this method is rarely invoked manually; under typical usage patterns,
the constructor of L<C<Aranea::XML::Aranea>|Aranea::XML::Aranea> will
call this method automatically.

=cut
sub new {
    my $proto = shift;
    my $data = shift;
    my $class = ref($proto) || $proto;
    my $self = $data;

    if ( $self->{entry} ) {
      map { Aranea::XML::RequestEntry->new($_) } @{$self->{entry}};
    } else {
      $self->{entry} = [];
    }

    #print Dumper($self);

    bless ($self, $class);
    return $self;
}
#>

#< sub entries

=item B<C<entries([$listref])>>

Accesses or mutates the collection of
L<C<RequestEntry>s|Aranea::XML::RequestEntry>.  When called with no
arguments, this method returns a listref of C<RequestEntry>s.  When
called with a single listref, the collection of C<RequestEntry>s is
set to the listref.

The typical usage pattern is to iterate over each request and perform
some action, e.g.,

  my $request_list = $Aranea->request_list();
  my $entries = $request_list->entries();
 
  for my $request ( @$entries ) {
    # do stuff...
  }

=cut
sub entries {
    my $self = shift;
    return $self->{entry};
}
#>
#< sub entries_count

=item B<C<entries_count()>>

Returns the number of entries in the C<RequestList>. Note that the
return value is similiar to that of C<$#>, i.e., -1 for no entries, 0
for one entry, etc.

=cut
sub entries_count {
    my $self = shift;
    return $#{@{$self->{entry}}};
}
#>
#< sub add_default_exact_query

=item B<C<add_default_exact_query($q)>>

Adds a new exact query to the C<RequestList> with default parameters.
An exact query specifies an unbound variable, e.g., execution of the
request "?x killed Lincoln" will return a small fragment of text that
directly preceeds the exact phrase "killed Lincoln."

=cut
sub add_default_exact_query {
    my $self = shift;
    my $q = shift;
    my $n = shift;

    my $cnt = $n ? $n : 100;

    # since we're inside the RequestList object, it's okay to
    # manipulate hashrefs directly (without violating encapsulation).
    # The RequestList and Request objects are tightly coupled anyway.

    my $request = {};
    $request->{query} = $q;
    $request->{score} = 5;

    $request->{constraints}->{max_length} = 50;
    $request->{constraints}->{max_words} = 10;

    $request->{constraints}->{pages_to_search} = $cnt;

    # now create the request
    my $robj = Aranea::XML::RequestEntry->new($request);

    push @{$self->{entry}}, $robj;
}
#>
#< sub add_default_inexact_query

=item B<C<add_default_inexact_query($q)>>

Adds a new inexact query to the C<RequestList> with default
parameters.  An inexact query I<does not> specifies an unbound
variable; rather, the surrounding context of the matched keywords is
returned.

=cut
sub add_default_inexact_query {
    my $self = shift;
    my $q = shift;
    my $n = shift;

    my $cnt = $n ? $n : 100;

    # since we're inside the RequestList object, it's okay to
    # manipulate hashrefs directly (without violating encapsulation).
    # The RequestList and Request objects are tightly coupled anyway.

    my $request = {};
    $request->{query} = $q;
    $request->{score} = 1;

    $request->{constraints}->{use_backoff} = 'true';
    $request->{constraints}->{pages_to_search} = $cnt;

    # now create the request
    my $robj = Aranea::XML::RequestEntry->new($request);

    push @{$self->{entry}}, $robj;
}
#>
#< sub write_xml

=item B<C<write_xml($writer)>>

Writes the XML representation of this C<RequestList>; takes an
C<XML::Writer> as its only argument.  Note that this method is rarely
invoked manually; under typical usage patterns, the C<write_xml>
method of L<C<Aranea::XML::Aranea>|Aranea::XML::Aranea> will call this
method automatically.

=cut
sub write_xml {
    my $self = shift;
    my $writer = shift;

    my $entries = $self->entries();

    $writer->startTag('request_list');
    for my $entry (@$entries) {
	$entry->write_xml($writer);
    }
    $writer->endTag('request_list');
}
#>

1;
